(ns wanijo.infrastructure.neo4j
  (:require [neo4j-clj.core :as db]
            [wanijo.infrastructure.devmode :as devmode]
            [clj-time.format :as time-format]
            [clj-time.local :as time-local]
            [clojure.spec.alpha :as spec]
            [clojure.string :as cljs])
  (:import (java.util UUID)))

(spec/def ::date-str
  #(re-matches #"\d{4}\d{2}\d{2}T\d{2}\d{2}\d{2}\.\d{3}Z" %))

(spec/def ::uuid
  #(re-matches
    #"[0-9a-f]+-[0-9a-f]+-[0-9a-f]+-[0-9a-f]+-[0-9a-f]+" %))

(defn create-conn!
  ([]
   (let [env #(or (System/getenv %1) %2)
         port (env "NEO4J_PORT" "7687")
         host (env "NEO4J_HOST" "localhost")
         user (env "NEO4J_USER" "neo4j")
         pass (env "NEO4J_PASS" "b")]
     (create-conn! port host user pass)))
  ([port host user pass]
   (db/connect
    (str "bolt://" host ":" port)
    user
    pass)))

(def conn
  (atom nil))

(defn reset-conn!
  ([]
   (reset! conn (create-conn!)))
  ([port host user pass]
   (reset! conn (create-conn! port host user pass))))

(defmacro defquery [& args] `(db/defquery ~@args))

(defn uuid []
  (str (UUID/randomUUID)))

(defn butiful-query [qry]
  (->> qry
       str
       cljs/trim-newline
       cljs/split-lines
       (map cljs/trim)
       (filter #(pos? (count %)))
       (cljs/join \newline)))

(defn exec-query! [qry params]
  (let [live-conn (if (nil? @conn)
                    (reset-conn!)
                    @conn)]
    (with-open [session (db/get-session live-conn)]
      (devmode/send-to-bar
       (str (butiful-query qry)
            "<br>---Params---<br>"
            params))
      (qry session params))))

(spec/def ::tuple-query-list
  (spec/coll-of
   (spec/tuple fn? map?)))
(defn exec-queries! [& tuples]
  {:pre [(spec/assert ::tuple-query-list tuples)]}
  (db/with-transaction @conn tx
    (doseq [tuple tuples]
      (let [qry (first tuple)
            params (second tuple)]
        (devmode/send-to-bar
         (str (butiful-query qry)
              "<br>---Params---<br>"
              params))
        (spec/assert map? params)
        (qry tx params)))))

(defn now-str []
  (time-format/unparse
   (time-format/formatters :basic-date-time)
   (time-local/local-now)))
